// ChessBoard.cpp: implementation of the ChessBoard class.
//
//////////////////////////////////////////////////////////////////////

#include "ChessBoard.h"
#include "ChessPieceSet.h"
#include "ChessPiece.h"
#include "ChessMove.h"
#include <cstdio>
#include <cctype>
#include "debug.h"

#define ASCII2INT(n) n-97

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ChessBoard::ChessBoard()
{	
	dprintf(("entered ChessBoard::ChessBoard()\n"));
	//clearSquares();
	for(int row=0; row<NUM_ROWS; row++)
		for(int col=0; col<NUM_COLS; col++){			
			squares[row][col] = NULL;
		}
	dprintf(("leaving ChessBoard::ChessBoard()\n"));
}

//////////////////////////////////////////////////////////////////////

ChessBoard::~ChessBoard()
{
	clearSquares();	
}

//////////////////////////////////////////////////////////////////////

void ChessBoard::clearSquare(int row, char col)
{
	delete squares[row-1][ASCII2INT(col)];
	squares[row-1][ASCII2INT(col)] = NULL;

}

//////////////////////////////////////////////////////////////////////

void ChessBoard::clearSquares()
{
	for(int row=0; row<NUM_ROWS; row++)
		for(int col=0; col<NUM_COLS; col++){
			if(squares[row][col] != NULL) 
				delete squares[row][col];
			squares[row][col] = NULL;
		}

}

//////////////////////////////////////////////////////////////////////

bool ChessBoard::isValidMove(ChessMove *)// *move)
{
  return true;
}

//////////////////////////////////////////////////////////////////////


/*
** return a pointer into this->squares which indicates the original
** location if a piece moved according to <move>, assumes that
** <move> hasn't been applied to this board
** NOTE: this does not change anything in this class
**
*/

ChessBoardSquare* ChessBoard::findFromSquare(ChessMove *move,bool isBlack) const
{	
  //long row = move->getRow();
  //char col = move->getCol();
  long fromRow = move->getFromRow(); // aren't using these yet, but i probably should
  char fromCol = move->getFromCol();	
  ChessPieceList *pieceList;
  ChessPieceList::iterator i;
  ChessPieceSet *pieceSet;
  ChessPiece *queen;
  ChessPiece *king;
  
  if(isBlack) pieceSet = blackPieces;
  else pieceSet = whitePieces;
  
  switch(move->getPiece()){
  case ChessPiece::PAWN:     // pawns
    pieceList = pieceSet->getPawns();
    if(move->getIsCapture()){
      for(i=pieceList->begin(); i!=pieceList->end(); i++){
	if((abs((*i)->getCol() - move->getCol()) == 1)  && 
	   (abs((*i)->getRow() - move->getRow()) == 1))
	  return getSquare((*i)->getRow(), (*i)->getCol());
      }
    }
    else{
      for(i=pieceList->begin(); i != pieceList->end(); i++){
	if((*i)->isOnCol(move->getCol()) && 
	   ((abs((*i)->getRow() - move->getRow()) == 1) || 
	    (abs((*i)->getRow() - move->getRow()) == 2)))
	  return getSquare((*i)->getRow(), (*i)->getCol());
      }	
    }
    break;
  case ChessPiece::ROOK:			// rooks
    pieceList = pieceSet->getRooks();
    for(i=pieceList->begin(); i != pieceList->end(); i++){
      fromRow = (fromRow == -1) ? (*i)->getRow() : fromRow;
      fromCol = (fromCol == -1) ? (*i)->getCol() : fromCol;
      if((*i)->isOnRow(fromRow) && (*i)->isOnRow(move->getRow())){
	return getSquare((*i)->getRow(), (*i)->getCol());
      }
      else if ((*i)->isOnCol(fromCol) && (*i)->isOnCol(move->getCol())){
	return getSquare(fromRow, fromCol);
      }
      fromRow = (fromRow == (*i)->getRow()) ? -1 : fromRow;
      fromCol = (fromCol == (*i)->getCol() ) ? -1 : fromCol;
    }
    break;
  case ChessPiece::KNIGHT:		// knights
    pieceList = pieceSet->getKnights();
    for(i=pieceList->begin(); i!=pieceList->end(); i++){
      if((abs((*i)->getCol() - move->getCol()) == 2) && 
	 (abs((*i)->getRow() - move->getRow()) == 1))
	return getSquare((*i)->getRow(), (*i)->getCol());
      else if((abs((*i)->getRow() - move->getRow()) == 2) && 
	      (abs((*i)->getCol() - move->getCol()) == 1))
	return getSquare((*i)->getRow(), (*i)->getCol());
    }
    break;
  case ChessPiece::BISHOP:		// bishops
    pieceList = pieceSet->getBishops();
    for(i=pieceList->begin(); i!=pieceList->end(); i++){
      if(abs((*i)->getRow() - move->getRow()) ==
	 abs((*i)->getCol() - move->getCol()))
	return getSquare((*i)->getRow(), (*i)->getCol());
    }
    break;
  case ChessPiece::QUEEN:			// queen
    queen = pieceSet->getQueen();
    return getSquare(queen->getRow(), queen->getCol());
    break;
  case ChessPiece::KING:			// king
    king = pieceSet->getKing();
    return getSquare(king->getRow(), king->getCol());
    break;
  default: // unknown piece...?
    break;
  }	
  
  return NULL; // should never get here if the move is valid on this board
}


//////////////////////////////////////////////////////////////////////

/*
** move some piece in one of the ChessPieceSets according to <move>
**
*/
	
/*
**
**	
*/
bool ChessBoard::movePiece(ChessMove *move, bool isBlackPiece)
{	
  ChessPieceSet *pieceSet = isBlackPiece ? blackPieces : whitePieces;
  
  // handls special cases first, i.e. check, checkMate, long and short castle
  if(move->getIsShortCastle()){
    // ugly, ugly, ugly, ugly
    ChessPiece *king = pieceSet->getKing();
    ChessPiece *rook = NULL; // get the rook on the proper side
    ChessPieceList::iterator i;
    for(i=pieceSet->getRooks()->begin() ; i!=pieceSet->getRooks()->end(); i++){
      if((*i)->isOnCol('h')) rook = (*i);
    }
    
    if(rook == NULL) return false;
    ChessPiece *tempPiece;
    tempPiece = new ChessPiece((*king));
    king->setSquare(rook->getRow(), rook->getCol());
    rook->setSquare(tempPiece->getRow(), tempPiece->getCol());
    
    //ChessBoardSquare *kingSquare = getSquare(king->getRow(), king->getCol());
    
    //ChessBoardSquare *rookSquare = getSquare(rook->getRow(), rook->getCol());
    
    delete tempPiece;
    
  }else if(move->getIsLongCastle()){
    ;	
  }else{
    
    // find the fromSquare which <move> refers to
    ChessBoardSquare *fromSquare = findFromSquare(move, isBlackPiece);
    
    
    if(fromSquare == NULL){
      dprintf(("  could not find from square\n"));
      return false;
    }else dprintf(("found from square: %d, %c\n", fromSquare->getRow(), fromSquare->getCol()));
    
    // if the move is a capture then remove the piece from the set
    
    ChessBoardSquare *toSquare = getSquare(move->getRow(), move->getCol());
    if(move->getIsCapture() && toSquare != NULL){   // if your capturing then the square
      pieceSet->removePiece(toSquare->piece);		// should have something in it,
      //pieceSet->removePiece(getSquare(move->getRow(), move->getCol()));
    }
    
    // change the row and col in the piece of fromSquare
    fromSquare->piece->setSquare(move->getRow(), move->getCol());
  }
  
  // would be faster to just clear the fromSquare and then
  // reset the new one
  resetSquaresFromPieces();
  
  // if move is check or checkmate then do something
  
  return true;
  
}

//////////////////////////////////////////////////////////////////////

void ChessBoard::print()
{
  for(int row=NUM_ROWS-1; row>=0; row--){
    for(int col=0; col<NUM_ROWS; col++){					
      if(squares[row][col] != NULL){
	printf("%c ", squares[row][col]->isBlack ? 
	       tolower(squares[row][col]->piece->getType()) : // lower case if black piece
	       squares[row][col]->piece->getType());
      }
      else printf("- ");
    }
    printf("\n");
  }
}

//////////////////////////////////////////////////////////////////////

void ChessBoard::resetSquaresFromPieces()
{
  clearSquares();
  
  ChessPieceList::iterator i;
  
  // white pawns
  for(i=whitePieces->pawns->begin(); i!=whitePieces->pawns->end(); i++)
    setSquarePointer((*i), false);
  
  // black pawns
  for(i=blackPieces->pawns->begin(); i!=blackPieces->pawns->end(); i++)
    setSquarePointer((*i), true);
  
  // white rooks
  for(i=whitePieces->rooks->begin(); i!=whitePieces->rooks->end(); i++)
    setSquarePointer((*i), false);
  
  // black rooks
  for(i=blackPieces->rooks->begin(); i!=blackPieces->rooks->end(); i++)
    setSquarePointer((*i), true);
  
  // white knights
  for(i=whitePieces->knights->begin(); i!=whitePieces->knights->end(); i++)
    setSquarePointer((*i), false);
  
  // black knights
  for(i=blackPieces->knights->begin(); i!=blackPieces->knights->end(); i++)		
    setSquarePointer((*i), true);
  
  // white bishops
  for(i=whitePieces->bishops->begin(); i!=whitePieces->bishops->end(); i++)
    setSquarePointer((*i), false);
  
  // black bishops
  for(i=blackPieces->bishops->begin(); i!=blackPieces->bishops->end(); i++)		
    setSquarePointer((*i), true);
  
  
  // queens
  setSquarePointer(whitePieces->queen, false);
  setSquarePointer(blackPieces->queen, true);
  
  // kings
  setSquarePointer(whitePieces->king, false);
  setSquarePointer(blackPieces->king, true);
  
}

//////////////////////////////////////////////////////////////////////

bool ChessBoard::initialize()
{
  whitePieces = new ChessPieceSet();
  blackPieces = new ChessPieceSet();
  
  whitePieces->initialize(false);
  blackPieces->initialize(true);
  
  resetSquaresFromPieces();
  
  return true;
  
}

void ChessBoard::setSquarePointer(ChessPiece *piece, bool isBlack)
{
  //dprintf(("ChessBoard::setSquarePointer(%d, %d)\n", piece->getRow()-1, (piece->getCol()-97)));
  int row, col;
  row = piece->getRow() - 1; // this->squares uses 0-7 vs 1-8
  col = piece->getCol() - 97;
  
  // should probably check that this location doesn't already
  // point to something, i.e. != NULL
  //setSquare(piece->getRow(), piece->getCol(), piece, isBlack);
  
  ChessBoardSquare *square = new ChessBoardSquare;
  square->piece = piece;
  square->isBlack = isBlack;
  squares[row][col] = square;
}
