//
// ChessMove.cpp
//

#ifdef WIN32
#define _UNICODE // win32 unicode support 
#endif

#include "ChessMove.h"
#include "ChessPieceSet.h"
#include "JavaCPP.h"
#include "JObject.h"
#include "debug.h"
#include <cstdlib>  // itoa
#include <cctype>
#include "ScriptController.h"
#include "ChessPiece.h"

ChessMove::ChessMove()
{
  col = -1;
  row = -1;
  fromCol = -1;
  fromRow = -1;
  piece = -1;
  isCapture = false;
  isCheck = false;
  isCheckMate = false;
  isLongCastle = false;
  isShortCastle = false;
}

//////////////////////////////////////////////////////////////////////////////


ChessMove::ChessMove(char *move)
{
  dprintf(("entered ChessMove(%s)\n", move));
	
  // default values for members
  col = -1;
  row = -1;
  fromCol = -1;
  fromRow = -1;
  piece = -1;
  isCapture = false;
  isCheck = false;
  isCheckMate = false;
  isLongCastle = false;
  isShortCastle = false;
  
  if(strcmp(move, "O-O") == 0){
    isLongCastle = true;
    return;
  }
  if(strcmp(move, "O-O-O") == 0){
    isShortCastle = true;
    return;
  }
  
  int pos = 0;
  switch(move[pos]){		
  case ChessPiece::KNIGHT: 
    piece = ChessPiece::KNIGHT; pos++; break;
  case ChessPiece::ROOK:
    piece = ChessPiece::ROOK; pos++; break;
  case ChessPiece::BISHOP:
    piece = ChessPiece::BISHOP; pos++; break;
  case ChessPiece::QUEEN:
    piece = ChessPiece::QUEEN; pos++; break;
  case ChessPiece::KING:
    piece = ChessPiece::KING; pos++; break;
  default:
    piece = ChessPiece::PAWN;
    ; // piece must be pawn or the move is a castle
    break;
  }
  
  // need to handle case where there are two of the same piece on
  // the board so in the move you need to specify the piece
  // e.g. Qcc1 mean move the queen in col c to c1
  // ??? *** what about two pieces in the same row, Q3c2 ?	
  if(('a' <= move[pos] && move[pos] <= 'h') && strlen(move) >= 4){	
    dprintf(("  fromCol: %c\n", move[pos]));
    fromCol = move[pos];
    pos++;	    
  }
  
  if(('0' <= move[pos] && move[pos] <= '8') && strlen(move) >= 4){
    dprintf(("  fromRow: %c\n", move[pos]));
    fromRow = move[pos] - 48;
    pos++;		
  }
  
  
  if(move[pos] == 'x'){   // a capture
    dprintf(("  is capture\n"));
    isCapture = true;
    pos++;
  }
  
  
  col = move[pos++];		
  row = atoi(&move[pos++]);
  if(move[pos] == '#' || (move[pos] == '+' && move[pos+1] == '+')){
    isCheckMate = true;
    pos++;
  }else if(move[pos] == '+'){ 
    isCheck = true;
    pos++;
  }
  
  
  // *** TODO: parse these from the move stringshould 
  // what about getting a piece back e.g. c8=Q
  // using '=' is how GNU chess does it, find out how other
  // notation do it
  
  dprintf(("leaving ChessMove(%s)\n", move));
}

//////////////////////////////////////////////////////////////////////////////


ChessMove::~ChessMove()
{

}

//////////////////////////////////////////////////////////////////////////////

jobject ChessMove::toJNIObject(JavaCPP *jcpp)
{
  //dprintf(("entered ChessMove::toJNIObject()\n"));
  jstring moveStr = jcpp->getEnv()->NewStringUTF(toAlgeabraicMove());
  
  if(moveStr == NULL){
    return NULL;	
  }
  
  //dprintf(("  about to jcpp->newObject\n"));	
  jvalue value;
  value.l = moveStr;
  JObject *obj = jcpp->newObject("players/common/JChessMove", 
				 "(Ljava/lang/String;)V", moveStr);	
  if(obj == NULL || jcpp->getEnv()->ExceptionCheck()){
    dprintf(("ChessMove::toJNIObject(jcpp)\nCould not get a new"
	     "JChessMove JObject\n"))
      return NULL;
  }
  
  //dprintf(("leaving ChessMove::toJNIObject()\n"));
  return obj->getJNIObject();
}

//////////////////////////////////////////////////////////////////////////////


/*
**  TODO: this function hasn't been tested very thoroughly
**
*/
char* ChessMove::toAlgeabraicMove()
{
  //dprintf(("entered ChessMove::toAlgeabraicMove()\n"));
  
  if(isShortCastle) return "O-O";
  if(isLongCastle) return "O-O";
  
  char *str = new char[10];
  memset(str, 0, 10);
  int currStrPos = 0;
  
  //dprintf(("  piece: %s\n", piece == PAWN ? "PAWN" : &piece));
  
  if(piece != ChessPiece::PAWN) str[currStrPos++] = piece;
  
  if(fromCol != -1) str[currStrPos++] = fromCol;
  
  if(fromRow != -1) str[currStrPos++] = fromRow+48;
  
  if(isCapture) str[currStrPos++] = 'x';	
  
  str[currStrPos++] = col;
  
  //char buff[1];
  //_itoa(row, buff, 10);	
  //str[currStrPos++] = buff[0];
  str[currStrPos++] = row+48;
  
  
  if(isCheck) str[currStrPos++] = '+';
  else if(isCheckMate) str[currStrPos++] = '#';
  
  str[currStrPos] = '\0';
  
  //dprintf(("leaving ChessMove::toAlgeabraicMove(%s)\n", str));
  return str;
}

//////////////////////////////////////////////////////////////////////////////

