//
// ChessPieceSet.cpp
//


#include "chessdef.h"
#include "ChessPieceSet.h"
#include "ChessPiece.h"

const int ChessPieceSet::NUM_PAWNS = 8;
const int ChessPieceSet::NUM_ROOKS = 2;
const int ChessPieceSet::NUM_KNIGHTS = 2;
const int ChessPieceSet::NUM_BISHOPS = 2;
const int ChessPieceSet::NUM_QUEENS = 1;
const int ChessPieceSet::NUM_KINGS = 1;

ChessPieceSet::ChessPieceSet()
{
  pawns = NULL;
  rooks = NULL;
  knights = NULL;
  bishops = NULL;
  queen = NULL;
  king = NULL;
}

//////////////////////////////////////////////////////////////////////

ChessPieceSet::~ChessPieceSet()
{
  ChessPieceList::iterator i;
  SAFE_DELETE_LIST(i, pawns);  
  SAFE_DELETE_LIST(i, rooks);
  SAFE_DELETE_LIST(i, knights);
  SAFE_DELETE_LIST(i, bishops);
  SAFE_DELETE(queen);
  SAFE_DELETE(king);
}

//////////////////////////////////////////////////////////////////////

bool ChessPieceSet::initialize(bool isBlack)
{
  int row;
  char col;
  
  pawns = new ChessPieceList();
  rooks = new ChessPieceList();
  knights = new ChessPieceList();
  bishops = new ChessPieceList();
  
  // location for pawns
  if(isBlack){  // black pieces
    row = 7;
    col = 'a';
  } 
  else{   // white pieces
    row = 2;
    col = 'a';	
  }
  
  // create the pawns
  for(int i=0; i<NUM_PAWNS; i++)
    pawns->insert(pawns->begin(), new ChessPiece(ChessPiece::PAWN, row, col++));
  
  // row is determined by the color
  if(isBlack)	row = 8;
  else row = 1;
  
  // create all other pieces
  rooks->insert(rooks->begin(), new ChessPiece(ChessPiece::ROOK, row, 'a'));
  rooks->insert(rooks->begin(), new ChessPiece(ChessPiece::ROOK, row, 'h'));
  knights->insert(knights->begin(), 
		  new ChessPiece(ChessPiece::KNIGHT, row, 'b'));
  knights->insert(knights->begin(), 
		  new ChessPiece(ChessPiece::KNIGHT, row, 'g'));
  bishops->insert(bishops->begin(), 
		  new ChessPiece(ChessPiece::BISHOP, row, 'c'));
  bishops->insert(bishops->begin(), 
		  new ChessPiece(ChessPiece::BISHOP, row, 'f'));
  queen = new ChessPiece(ChessPiece::QUEEN, row, 'd');
  king = new ChessPiece(ChessPiece::KING, row, 'e');
  
  return true;
}

//////////////////////////////////////////////////////////////////////


bool ChessPieceSet::removePiece(ChessPiece *piece)
{
  switch(piece->getType()){
  case ChessPiece::PAWN:
    pawns->remove(piece);
    break;
  case ChessPiece::ROOK:
    rooks->remove(piece);
    break;
  case ChessPiece::KNIGHT:
    knights->remove(piece);
    break;
  case ChessPiece::BISHOP:
    bishops->remove(piece);
    break;
  case ChessPiece::QUEEN:
    delete queen;
    queen = NULL;
    break;
  case ChessPiece::KING:
    delete king;
    king = NULL;
    break;
  default:
    return false; // should never get here....?
    break;
  }
  
  return true;
}


//////////////////////////////////////////////////////////////////////
