// BlackPlayerAI.java

package players.gnuchess_win32;

import java.io.*;
import players.common.*;

class BlackPlayerAI implements ChessPlayerAI
{
    public BlackPlayerAI()
    {
	System.out.println("players.gnuchess_win32.BlackPlayerAI()\n");
    }       

    Process gnuChess;
    OutputStreamWriter outStream;
    BufferedReader inStream;
    
    String exeName = "./players/gnuchess_win32/gnuchess";

    public boolean initialize() throws IOException
    {
	System.out.println("entered BlackPlayerAI::initialized()\n");
	gnuChess = Runtime.getRuntime().exec(exeName);
	outStream = new OutputStreamWriter(gnuChess.getOutputStream());
	inStream = 
	    new BufferedReader(new InputStreamReader(gnuChess.getInputStream()));
	// "nopost" tells gnuchess not to think out loud
	outStream.write("nopost\n", 0, 7);  // don't think outloud	
	outStream.flush();
	return true;
    }

    public int shutDown() throws IOException, InterruptedException
    {
	if(outStream != null){
	    outStream.write("quit\n", 0, 5);
	    outStream.flush();
	    outStream.close();
	}
	if(inStream != null)   inStream.close();
	if(gnuChess != null){
	    gnuChess.waitFor();	   
	    return gnuChess.exitValue();	
	} 
	return 1;
    }

    public JChessMove getNextMove(JChessMove oppPrevMove) throws IOException
    {
	String oppString = (oppPrevMove == null ? 
			    null : oppPrevMove.toAlgeabraicMove());
	
	System.out.println("BlackPlayerAI.getNextMove(" + 
			   (oppString == null ? "null" : oppString) + ")");
	
	if(oppPrevMove != null){	  
	    System.out.println("  oppString: " + oppString + "., " + 
			       oppString.length());
	    outStream.write(oppString + '\n', 0, oppString.length()+1);	    
	    outStream.flush();
	}
	else{
	    System.out.println(" i'll go first...\n");
	    outStream.write("go\n", 0, 3);
	    outStream.flush();
	}
	
	String buffStr;
	boolean done = false;
	do{
	    buffStr = inStream.readLine();	    	    
	    if(buffStr.length() > 9)
		done = buffStr.startsWith("My move is");	    
	}while(!done);
	
	
	String moveStr = buffStr.substring(buffStr.lastIndexOf(':') + 2,
					   buffStr.length());	
	System.out.println("leaving BlackPlayerAI.getNextMove()");
	return new JChessMove(moveStr);
    }
}
