//
// ScriptController.cpp
//

#include "chessdef.h"
#include "ScriptController.h"
#include "ChessMove.h"
#include "JavaCPP.h"
#include "JObject.h"

#include "debug.h"


ScriptController::ScriptController(JavaCPP *jcpp, const string &className)
{
  dprintf(("entered ScriptController(%s, %s)\n", jcpp == NULL ? 
	   "**NULL**" : "jcpp", className.c_str()));
  initialized = false;
  this->jcpp = jcpp;
  this->className = className;	
  obj = NULL;
  getNextMoveID = NULL;
  dprintf(("leaving ScriptController()\n"));
}

//////////////////////////////////////////////////////////////////////////////

ScriptController::~ScriptController()
{	
  dprintf(("entered ~ScriptController()\n"));
  if(obj != NULL && obj->isOK()){
    jmethodID shutDownID = obj->getMethodID("shutDown", methodSigs.shutdown);
    if(shutDownID == NULL){
      if(jcpp->getEnv()->ExceptionCheck()) jcpp->getEnv()->ExceptionDescribe();
      dprintf(("  could not GetMethodID() for shutDown(): %s\n", 
	     className.c_str()));		
    }
    if(obj->callIntMethod(shutDownID) != 0){
      if(jcpp->getEnv()->ExceptionCheck()) jcpp->getEnv()->ExceptionDescribe();
      dprintf(("  problem shutting down %s\n", className.c_str()));
    }
  }
  
  SAFE_DELETE(obj);  
  dprintf(("leaving ~ScriptController()\n"));
}

//////////////////////////////////////////////////////////////////////////////


bool ScriptController::initialize()
{
  dprintf(("entered ScriptController::initialize()\n"));	
	
  obj = jcpp->newObject(className.c_str(), methodSigs.constructor, NULL);
  if(obj == NULL){
    dprintf(("  could not jcpp->newObject()\n"));
    return false;
  }	
  
  // get and store methodID for getNextMove	
  getNextMoveID = obj->getMethodID("getNextMove", methodSigs.getNextMove);
  
  if(getNextMoveID == NULL){
    dprintf(("  could not GetMethodID() for getNextMove(): %s\n", 
	     className.c_str()));
    return false;    
  }
  
  // get methodID for initialize and call the funtions
  jmethodID initID = obj->getMethodID("initialize", methodSigs.initialize);
  if(initID == NULL){
    dprintf(("  could not GetMethodID() for initialize(): %s\n", 
	     className.c_str()));
    return false;
  }
  
  dprintf((" about to call initialize\n"));	
  if(obj->callBooleanMethod(initID) == JNI_FALSE){
    dprintf(("  Error initializing Java class: %s\n", className.c_str()));
    return false;
    
  }	
  
  dprintf(("leaving ScriptController::initialize()\n"));
  return initialized = true;
}


//////////////////////////////////////////////////////////////////////////////


// send <oppPrevMove> to the Java class and the class
// will return it's move
ChessMove* ScriptController::getNextMove(ChessMove *oppPrevMove)
{
	
  if(!initialized) return NULL;
  
  //dprintf(("entered ScriptController::getNextMove()\n"));	
  JObject *move;
  if(oppPrevMove == NULL){
    dprintf(("  oppPrevMove == NULL\n"));
    move = obj->callObjectMethod(getNextMoveID, NULL);
  }	
  else {
    dprintf(("  oppPrevMove NOT = NULL\n"));		
    move = obj->callObjectMethod(getNextMoveID, oppPrevMove->toJNIObject(jcpp));
  }
    
  if(move == NULL) return NULL;	
    
  dprintf(("about to get methodID for toAlgeabraicMove\n"));
  // get the jstring from the returned ChessMove JObject
  // and pass the string of the jstring to ChessMove(str);
  jmethodID toStringID = move->getMethodID("toAlgeabraicMove", 
					   "()Ljava/lang/String;");
  if(toStringID == NULL) return NULL;  
  
  
  dprintf(("  getting algeabraic string from JChessMove\n"));
  jstring jstr = (jstring)(move->callObjectMethod(toStringID))->getJNIObject();
  
  dprintf(("  got JObject\n"));
  
  const char *str = jcpp->getEnv()->GetStringUTFChars(jstr, 0);
  dprintf(("  got it\n"));
  dprintf(("  str:  %s\n", str));	
  
  ChessMove *chessMove = new ChessMove((char *)str);
  
  // cleanup
  jcpp->getEnv()->ReleaseStringUTFChars(jstr, str);
  delete move;  // no longer need JObject move
  
  // leave
  dprintf(("leaving ScriptController::getNextMove()\n"));
  return chessMove;
  
}
