//debug.cpp
//CDebugManager functions

//Copyright 1999, Ian Parberry.  All rights reserved.

#include "debug.h"

//ensure debug code is not compiled unless needed
#if defined(DEBUG_ON) && defined(WIN32) 

CDebugManager::CDebugManager(){ //constructor
  #ifdef DEBUG_OUTPUT_TO_FILE
    m_fileHandle=fopen(DEBUG_FILENAME,"wt"); //open output file
  #endif
  #ifdef DEBUG_OUTPUT_TO_COMPORT
    m_cSerialPort=new CSerialPort(DEBUG_SERIALPORT);
  #endif
}

CDebugManager::~CDebugManager(){ //destructor
  #ifdef DEBUG_OUTPUT_TO_FILE
    fclose(m_fileHandle); //close output file
  #endif
  #ifdef DEBUG_OUTPUT_TO_COMPORT
    delete m_cSerialPort;
  #endif
}

void CDebugManager::setsource(char* file,int line){ //set file name and line number
	char* p=strrchr(file,'\\');
  if(p==NULL)strcpy(m_szFileName,file); //set file name
	else strcpy(m_szFileName,(char*)(p+1));
  m_nLineNumber=line; //set line number
}

void CDebugManager::printf(const char* format,...){ //debug printf
  //print file and line number to m_szOutBuffer
  sprintf(m_szOutBuffer,"%s(%d): ",m_szFileName,m_nLineNumber);
  //append debug message from parameter list
  va_list arglist;
  va_start(arglist,format);
  vsprintf((char*)(m_szOutBuffer+strlen(m_szOutBuffer)),format,arglist);
  va_end(arglist);
  //print m_szOutBuffer to file if needed
  #ifdef DEBUG_OUTPUT_TO_FILE
    fprintf(m_fileHandle,"%s",m_szOutBuffer);
    fflush(m_fileHandle);
  #endif
  //print m_szOutBuffer to debugger if needed
  #ifdef DEBUG_OUTPUT_TO_DEBUGGER
    OutputDebugString(m_szOutBuffer);
  #endif
  //print m_szOutBuffer to COM port if needed
  #ifdef DEBUG_OUTPUT_TO_COMPORT
    m_cSerialPort->WriteCommBlock(m_szOutBuffer,strlen(m_szOutBuffer));
    m_cSerialPort->WriteCommBlock("\r",1);
  #endif
  #ifdef DEBUG_OUTPUT_TO_IP
    m_cIPManager.SendPacket(m_szOutBuffer,strlen(m_szOutBuffer));
  #endif
}

#endif
