//debug.h
//Include this file everywhere that you want debug output
//except for your main - include debugdec.hpp there instead.

#include "debugset.h" //debug settings

//declaration of debug manager class CDebugManager

#ifndef __DEBUG_H__ //to prevent compiler seeing this twice
#include <cstdio>

//ensure debug code is not compiled unless needed
#if defined(DEBUG_ON) && defined(WIN32)

#include <windows.h>


#ifdef DEBUG_OUTPUT_TO_COMPORT
#include "SerialPort.h"
#endif

#ifdef DEBUG_OUTPUT_TO_IP
#include "IPMgr.h"
#endif

#define DEBUG_OUTBUF_SIZE 1024 //size of output buffer
#define DEBUG_FNAME_SIZE 64 //size of file name

class CDebugManager{ //the debug manager
  private:
    FILE *m_fileHandle; //handle for debug output file
    char m_szOutBuffer[DEBUG_OUTBUF_SIZE]; //output buffer
    char m_szOutBuffer2[DEBUG_OUTBUF_SIZE]; //another output buffer
    char m_szFileName[DEBUG_FNAME_SIZE]; //file that debug message comes from
    int m_nLineNumber; //line that debug message comes from
#ifdef DEBUG_OUTPUT_TO_COMPORT
		CSerialPort* m_cSerialPort; //serial port for debug output
#endif
#ifdef DEBUG_OUTPUT_TO_IP
    IPManager m_cIPManager;
#endif
  public:
    CDebugManager(); //constructor
    ~CDebugManager(); //destructor
    void printf(const char* format,...); //debug printf
    void setsource(char* file,int line); //set file and line number
	FILE *getFileHandle(){ return m_fileHandle; };
};

#endif //DEBUG_ON
#endif //__DEBUG_H__

//Copyright 1999, Ian Parberry.  All rights reserved.

//external declaration of debug manager

#if defined(DEBUG_ON) && defined(WIN32)
  extern CDebugManager g_cDebugManager; //defined in debugman.hpp
#endif //DEBUG_ON

//global function calls used for debugging
#ifndef __DEBUG_H__ //to prevent compiler seeing this twice

#if defined(DEBUG_ON) && defined(WIN32)
//debug printf
#define dprintf(A) \
{ \
  g_cDebugManager.setsource(__FILE__,__LINE__); \
  g_cDebugManager.printf A; \
}

#elif defined(DEBUG_ON)
#define dprintf(A) printf A;

#else
#define dprintf(A) //nothing

#endif //DEBUG_ON

//final defines
#define __DEBUG_H__ //this must be the last define in this file

#endif //__DEBUG_H__
