//
// JObject.h
//


#ifndef _JOBJECT_H_
#define _JOBJECT_H_

#include <jni.h>

// this class cannot be directly instantiated but must be created
// via the iJavaCPP interface

class JObject  
{
protected:
  JObject(JNIEnv *env, const char *className, const char *contructorSig, 
	  va_list args);
  // assumes that <obj> has already been "newed" somewhere
  // else and its contructor already called
  JObject(JNIEnv *env, jobject obj); 	
  
  friend class iJavaCPP; // allow iJavaCPP to be our factory
  
public:	
  virtual ~JObject();
  
  // get the methodID for a function with name <methodName>
  // and signature <methodSig>
  jmethodID getMethodID(char *methodName, char *methodSig);
  
  // get the field which corresponds with <fieldName>
  // and signature <fieldID>
  
  // NOTE: getObjectField() return a JObject* versus the
  // regular jobject as return by the JNI equivalent
  JObject* getObjectField(char *methodName, char *methodSig);
  jboolean getBooleanField(char *methodName, char *methodSig);
  jbyte getByteField(char *methodName, char *methodSig);
  jchar getCharField(char *methodName, char *methodSig);
  jshort getShortField(char *methodName, char *methodSig);
  jint getIntField(char *methodName, char *methodSig);
  jlong getLongField(char *methodName, char *methodSig);
  jfloat getFloatField(char *methodName, char *methodSig);
  jdouble getDoubleField(char *methodName, char *methodSig);
  
  // call the appropriate method depending on the return type
  
  // NOTE: callObjectMethod() return a JObject* versus the 
  // regular jobject as returned by the JNI equivalent
  JObject *callObjectMethod(jmethodID methodID, ...);	
  void callVoidMethod(jmethodID methodID, ...);	
  jboolean callBooleanMethod(jmethodID methodID, ...);
  jbyte callByteMethod(jmethodID methodID, ...);
  jchar callCharMethod(jmethodID methodID, ...);
  jshort callShortMethod(jmethodID methodID, ...);
  jint callIntMethod(jmethodID methodID, ...);
  jlong callLongMethod(jmethodID methodID, ...);
  jfloat callFloatMethod(jmethodID methodID, ...);
  jdouble callDoubleMethod(jmethodID methodID, ...);
  
  
  // member accessors
  jclass getClass(){return clazz;};
  jobject getJNIObject(){ return object; };
  char *getClassName(){ return className; }

  bool isOK() const { return OK; };

protected:	
  
  bool OK;
  jobject object;
  jclass clazz;
  JNIEnv *env;
  char *className;
};

#endif // _JOBJECT_H_
