//
// JavaCPP_win32.cpp
//


#include "JavaCPP_win32.h"
#include <windows.h>
#include <sys/types.h>
#include <sys/stat.h>

// temporarily disable debugging
//#include "debug.h"
#define dprintf(N)

#define	RELEASE_1_3 "1.3"
#define	RELEASE_1_2 "1.2"
#define JRE_KEY	    "Software\\JavaSoft\\Java Runtime Environment"
#define JVM_DLL		"jvm.dll"
#define JAVA_DLL	"java.dll"


//////////////////////////////////////////////////////////////////////////////


JavaCPP_win32::JavaCPP_win32()
{
	jvmLib = NULL;
}

//////////////////////////////////////////////////////////////////////////////

JavaCPP_win32::~JavaCPP_win32()
{	
	dprintf(("entered ~iJavaCPP_win32()\n"));
	FreeLibrary(jvmLib);	
	dprintf(("leaving ~iJavaCPP_win32()\n"));
}


//////////////////////////////////////////////////////////////////////////////

bool JavaCPP_win32::getJNIFuncHandles()
{
	// get the base path of the library
	if(!getJREHome(jreHome, sizeof(jreHome))) return false;
	dprintf((" ** %s\n", jreHome));

	if(getJVMType() < 0) return false;
	dprintf(("  jvmType: %s\n", jvmType));

	// load library	
	char buf[256];	
	sprintf(buf, "%s\\bin\\%s\\jvm.dll", jreHome, jvmType);

	dprintf(("  loading library: %s\n", buf));
	jvmLib = LoadLibrary(buf);			

	jniFunctions.CreateJavaVM = (jint (JNICALL *)(JavaVM **, void **, void *))
		GetProcAddress(jvmLib, "JNI_CreateJavaVM");		

	return true;
}



//////////////////////////////////////////////////////////////////////////////

// return whether we have to hotspot or
// classic virtual machine
int JavaCPP_win32::getJVMType()
{
	dprintf(("entered JavaCPP_win32::getJVMType()\n"));
	
	struct stat s;

	sprintf(jvmDll, "%s\\bin\\hotspot\\%s", jreHome, JVM_DLL);	
	if (stat(jvmDll, &s) == 0){		
		strncpy(jvmType, "hotspot\0", 8);
		return iJavaCPP::JVM_HOTSPOT;
	}

	sprintf(jvmDll, "%s\\bin\\classic\\%s", jreHome, JVM_DLL);		
	if (stat(jvmDll, &s) == 0) {		
		strncpy(jvmType, "classic\0", 8);		
		return iJavaCPP::JVM_CLASSIC;
	}

	/*
	if (jvmType == NULL) {
		//sprintf(jvmDll, "%s\\bin\\hotspot\\" JVM_DLL, home);
		sprintf(jvmDll, "%s\\bin\\hotspot\\%s", jreHome, JVM_DLL);
		if (stat(jvmDll, &s) == 0) return iJavaCPP::JVM_HOTSPOT;
		sprintf(jvmDll, "%s\\bin\\classic\\%s", jreHome, JVM_DLL);		
		if (stat(jvmDll, &s) == 0) return iJavaCPP::JVM_CLASSIC;
	} else {
		sprintf(jvmDll, "%s\\bin\\%s\\%s", jreHome, jvmType, JVM_DLL);
		return iJavaCPP::JVM_OTHER;
	}*/
		
	dprintf(("leaving JavaCPP_win32::getJVMType(): -1"));
	return -1;

}


//////////////////////////////////////////////////////////////////////////////

bool getStringFromRegistry(HKEY key,const char *name, char *buf, jint bufsize)
{
    DWORD type, size;

    if (RegQueryValueEx(key, name, 0, &type, 0, &size) == 0
		&& type == REG_SZ && (size < (unsigned int)bufsize)) {
		if (RegQueryValueEx(key, name, 0, 0, (unsigned char *)buf, &size) == 0) {
			return JNI_TRUE;
		}
    }
    return JNI_FALSE;
}

//////////////////////////////////////////////////////////////////////////////

/*
** NOTE: set this up so the JRE or JVM can
** be in diff't locations
*/

bool JavaCPP_win32::getJREHome(char *buf, int bufsize)
{
	HKEY key, subkey;
	char version[256];
	

	// get the currentversion of the jre
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, JRE_KEY, 0, KEY_READ, &key) != 0) {
		fprintf(stderr, "Error opening registry key '" JRE_KEY "'\n");
		dprintf(("Error opening registry key '" JRE_KEY "'\n"));
		return JNI_FALSE;
    }

	if (!getStringFromRegistry(key, "CurrentVersion", version, sizeof(version))) {
		fprintf(stderr, "Failed reading value of registry key:\n\t"
			JRE_KEY "\\CurrentVersion\n");
		dprintf(("Failed reading value of registry key:\n\t"
			JRE_KEY "\\CurrentVersion\n"));
		RegCloseKey(key);
		return JNI_FALSE;
    }

	if (strcmp(version, RELEASE_1_3) != 0 && strcmp(version, RELEASE_1_2) != 0) {
		fprintf(stderr, "Registry key '" JRE_KEY "\\CurrentVersion'\nhas "
			"value '%s', but '" RELEASE_1_2 "' or greater is required.\n", version);
		dprintf(("Registry key '" JRE_KEY "\\CurrentVersion'\nhas "
			"value '%s', but '" RELEASE_1_2 "' or greater is required.\n", version));
		RegCloseKey(key);
		return JNI_FALSE;
    }


	/* Find directory where the current version is installed. */
    if (RegOpenKeyEx(key, version, 0, KEY_READ, &subkey) != 0) {
		fprintf(stderr, "Error opening registry key '"
			JRE_KEY "\\%s'\n", version);
		dprintf(("Error opening registry key '" JRE_KEY "\\%s'\n", version));
		RegCloseKey(key);
		return JNI_FALSE;
    }

    if (!getStringFromRegistry(subkey, "JavaHome", buf, bufsize)) {
		fprintf(stderr, "Failed reading value of registry key:\n\t"
			JRE_KEY "\\%s\\JavaHome\n", version);
		dprintf(("Failed reading value of registry key:\n\t"
			JRE_KEY "\\%s\\JavaHome\n", version));
		RegCloseKey(key);
		RegCloseKey(subkey);
		return JNI_FALSE;
    }

	/*
	// Determine if Hotspot VM is installed.
	char jvmType[256];
	sprintf(jvmdll, "%s\\bin\\hotspot\\" JVM_DLL, home);
	if(stat(jvmdll, &s) < 0) {
	    jvmType = "classic";
	} else {
	    jvmType = "hotspot";
	}
    */

	RegCloseKey(key);
  RegCloseKey(subkey);
	return true;

}


	
