//
// iJavaCPP.cpp
//


#include "iJavaCPP.h"
#include "JObject.h"
#include <cstring>
#include <cstdarg>

#include "debug.h"

#define SAFE_DELETE(thing) if(thing){ delete thing; } thing = NULL;

//////////////////////////////////////////////////////////////////////////////

const int iJavaCPP::JVM_NONE	= -1;
const int iJavaCPP::JVM_HOTSPOT = 0;
const int iJavaCPP::JVM_CLASSIC = 1;
const int iJavaCPP::JVM_OTHER   = 2;

//////////////////////////////////////////////////////////////////////////////

iJavaCPP::iJavaCPP()
{
  jvm = NULL;
  env = NULL;
  reset();  
}

//////////////////////////////////////////////////////////////////////////////

iJavaCPP::~iJavaCPP()
{
  dprintf(("entered ~iJavaCPP()\n"));
  reset();  
  dprintf(("leaving ~iJavaCPP()\n"));
}

void iJavaCPP::reset(){
  dprintf(("entered iJavaCPP::reset()\n"));
  dprintf(("*** if you are using the IBM JDK/JRE then it will hang here()\n"));
  
  if(jvm){
    jvm->DetachCurrentThread(); 
    jvm->DestroyJavaVM();
  }
  SAFE_DELETE(env);
  SAFE_DELETE(jvm); // should i do this
  dprintf(("leaving iJavaCPP()::reset()\n"));
  
}

//////////////////////////////////////////////////////////////////////////////

/*
** intialize a VM with <options> and create the VM if one 
** doesn't already exit
*/

bool iJavaCPP::init(JavaVMOption *options, int nOptions)
{
  dprintf(("entered iJavaCPP::initialize()\n"));

  if(jvm){
    dprintf(("  the jvm has already been initialized\n"));
    return true;   // jvm has already been initialized
  }	
    
  memset(&vm_args, 0, sizeof(vm_args));
  vm_args.version = JNI_VERSION;  // specify vm_args version num
  vm_args.options = options;	
  vm_args.nOptions = nOptions;	
  vm_args.ignoreUnrecognized = JNI_FALSE;
  
  for(int i =0; i<nOptions; i++){
    dprintf(("    %d: %s\n", i, vm_args.options[i].optionString));
  }
  
  if(!getJNIFuncHandles()) return false;
  
  //-----------------------------------------------------	
  // TODO: check if a JVM already exists and if so then
  // just attach this to it as another thread
  //-----------------------------------------------------	
  
  if(jniFunctions.CreateJavaVM == NULL){		
    dprintf(("  could not get a handle to JNI_CreateJavaVM()\n"));
    return false;		
  }
  
  jint r = jniFunctions.CreateJavaVM(&jvm, (void **)&env, &vm_args);	
  dprintf(("leaving iJavaCPP::createJavaVM()\n"));
  
  
  return r == JNI_OK ? OK = true : false;
  
}


//////////////////////////////////////////////////////////////////////////////


/*
** get the class and object named className and call
** the contructor defined by the signature
** return the jobject
** (some code borrowed from Sun's java.c java launcher)
*/

JObject* iJavaCPP::newObject(const char *className, const char *constuctorSig,
			     ...)
{
  if(!isOK()) return NULL;
  
  va_list args;
  va_start(args, constuctorSig);
  dprintf(("  about to env->NewObject()\n"));	
  JObject *obj = new JObject(this->env, className, constuctorSig, args);
  va_end(args);
  
  
  if(obj->getJNIObject() == NULL) return NULL;
  return obj;
}
